#!/bin/python
#-*- coding: utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import os
os.environ['GI_TYPELIB_PATH'] = 'libcdosnotify:' + os.environ.get('GI_TYPELIB_PATH', '')

import gi
gi.require_version('CdosNotify', '1.0')
gi.require_version("Gtk", "3.0")
from gi.repository import CdosNotify
from gi.repository import Gtk,GLib,Gio,Gdk

CdosNotify.init('cdos-update')

import dbus
import dbus.service
from dbus.mainloop.glib import DBusGMainLoop
import apt

import sys
import traceback
import tempfile
import thread
import logging
import time
from datetime import datetime
from datetime import timedelta
from threading import Timer

from patch_lib import Result
from patch_lib import determineCheck
from patch_lib import getSleeptimeBeforeNextCheck
from patch_lib import setLastCheckTime

from app_update_type import AppUpdateTypeDB

logging.basicConfig(format= '%(levelname)s %(asctime)-15s %(message)s file:%(filename)s line:%(lineno)d', filename=tempfile.mktemp("cdos-back-update.log"),level=logging.DEBUG)

global uidir
global timer_id
uidir = sys.path[0] + "/ui"
timer_id = -1

global dbus_object
dbus_object = None

global autoinstall, autodownload, willcheckupdate, dayfreq, hourfreq,check_last_time,install_last_time
willcheckupdate = True
dayfreq = 0
hourfreq = 0

global last_update_cache_time
last_update_cache_time = None

global update_info
update_info = {}

global appupdatemode
appupdatemode = "DEFAULT_UPDATE"

global app_update_type_db_obj
app_update_type_db_obj = AppUpdateTypeDB(os.path.join(os.getenv("HOME"), ".config/cdos-update/app_update_type.db"))

global app_update_type_json
app_update_type_json = app_update_type_db_obj.get_debname_with_type_and_process_name()

import gettext as gettext1
gettext1.bindtextdomain('cdosupdate', '/usr/share/locale')
gettext1.textdomain('cdosupdate')
_ = gettext1.gettext

class PatchStatus:
    INITED = 0
    DOWNLOADING = 10
    DOWNLOADED = 14
    ALLDOWNLOADED = 15
    UPDATING = 20
    VERIFYING = 22
    INSTALLING = 23
    INSTALLED = 25
    ALLINSTALLED = 30
    ERROR_INSTALL = 51

def enter_update_call(notify_obj, action_name, data1, data2):
    logging.info("enter_update_call")
    notify_obj.close()
    os.system("/usr/bin/python /usr/share/cdos-update/frontend/check_updates.py &")

def handle_upgradeStatusChanged(statusCode, status_text, paramList):
    logging.info("handle_upgradeStatusChanged")
    def dist_upgrade(upgrade_cmd):
        logging.info( upgrade_cmd)
        os.system("bash " + upgrade_cmd)

    if statusCode == 100:
        try:
            upgrade_tool_main_path = os.path.join(paramList["upgrade_tool_dir"], "distupgrade")
            thread.start_new_thread(dist_upgrade, (upgrade_tool_main_path, ))
            time.sleep(1)
        except Exception as e:
            logging.error("handle_upgradeStatusChanged Exception:" + str(e))
        finally:
            pass

def cancel_update_call(notify_obj, action_name, data1, data2):
    logging.info("cancel_update_call")
    notify_obj.close()
    # if cancel, but check upgrade is ready, do upgrade
    global update_info
    global dbus_object
    def dist_upgrade(upgrade_cmd):
        logging.info("upgrade_cmd:" + upgrade_cmd)
        os.system("bash " + upgrade_cmd)
    # if is upgrading
    if ( "_upgrade" in update_info) and ("upgrading" in update_info["_upgrade"]) and (update_info["_upgrade"]["upgrading"]) and ( "upgrade_tool_dir" in update_info["_upgrade"]) and ( not update_info["_upgrade"]["upgrade_tool_dir"] == ""):
        try:
            # print "distupgrade"
            upgrade_tool_main_path = os.path.join( update_info["_upgrade"]["upgrade_tool_dir"], "distupgrade")
            # print "upgrade_tool_main_path:" + upgrade_tool_main_path
            thread.start_new_thread(dist_upgrade, (upgrade_tool_main_path, ))
            time.sleep(1)
        except Exception as e:
            print e
            logging.error("cancel_update_call1:" + str(e))
        finally:
            return
    # if no upgrading but you can upgrade
    if ( "_upgrade" in update_info) and ("status" in update_info["_upgrade"]) and (update_info["_upgrade"]["status"] == True):
        try:
            # print "can_dist_upgrade"
            dbus_object.can_dist_upgrade()
        except Exception as e:
            # print e
            logging.error("cancel_update_call2" + str(e))
        finally:
            return
    return

def checkmainwindow():
    try:
        session_bus = dbus.SessionBus()
        remote_object_2 = session_bus.get_object("org.cdos.update.ui", "/org/cdos/update/ui" )
        client_iface = dbus.Interface(remote_object_2, "org.cdos.update.ui")
        if client_iface.show() == True:
            return True
        return False
    except Exception as e:
        return False

def get_pid_and_process_name():
    pid_process_json = {}
    for dir_name in os.listdir("/proc"):
        cmdline_file = os.path.join( os.path.join("/proc", dir_name), "cmdline")
        if dir_name.isdigit() and os.path.isfile(cmdline_file):
            try:
                pid_cmd_fd = open( cmdline_file)
                pid_cmd = pid_cmd_fd.read()
                pid_cmd_fd.close()
                if pid_cmd.endswith("\x00"):
                    pid_cmd = pid_cmd[:-1]
                pid_process_json[pid_cmd] = dir_name
            except Exception as e:
                continue
    return pid_process_json

def updateapps(update_info, app_update_type_json):
    dialogui_show_flag = False
    system_bus = dbus.SystemBus()
    daemon_remote_object = system_bus.get_object("org.cdos.update", "/org/cdos/update" )
    daemon_iface = dbus.Interface(daemon_remote_object, "org.cdos.update")
    pid_process_json = get_pid_and_process_name()
    for debname in app_update_type_json:
        if ( not debname in update_info) and ( not debname in update_info["_deb_res"]["list"]):
            continue
        if app_update_type_json[debname]["update_type"] == "AUTO_UPDATE":
            print "update debname:" + debname
            metadata = {}
            metadata["package_name"] = debname
            metadata["version"] = ""
            metadata["pid"] = ""
            metadata["process_name"] = ""
            metadata["env_home"] = os.getenv("HOME")
            metadata["env_path"] = os.getenv("PATH")
            metadata["env_user"] = os.getenv("USER")
            metadata["env_display"] = os.getenv("DISPLAY")
            if False:
                metadata["exit_flag"] = False
            else:
                process_name = app_update_type_json[debname]["process_name"]
                if process_name in pid_process_json:
                    metadata["pid"] = pid_process_json[process_name]
                    metadata["process_name"] = process_name
                    metadata["exit_flag"] = True
                else:
                    metadata["exit_flag"] = False
            daemon_iface.installAppUpdate(metadata)

        if app_update_type_json[debname]["update_type"] == "AUTO_PROMPT_UPDATE":
            print "AUTO_PROMPT_UPDATE"
            dialogui_show_flag = True
    if dialogui_show_flag:
        global n
        n = CdosNotify.Notification.new(_('CDOS Update'), _('Download and install app updates?'), 'cdos-update')
        n.add_action('yes', _('yes'), enter_update_call, None, None)
        n.add_action('no', _('no'), cancel_update_call, None, None)
        n.show()

def refreshUpdateThread():
    logging.debug("refreshUpdateThread")
    global dbus_object
    global update_info
    global app_update_type_json

    canBeUpdate = False
    appCanBeUpdate = False
    try:
        repo_url  = "http://repos.01os.org/"
        site_path = repo_url + "data/cdos-update/update/"
        getpatchlistinfo_site_patch = site_path + "getpatchesinfoWithSystemVersion"
        upgrade_path = repo_url + "data/cdos-update/cdos-upgrade-new-server/meta-release"

        update_info = dbus_object.getAllUpdates(os.environ["HOME"] ,upgrade_path, getpatchlistinfo_site_patch)
        if update_info["_upgrade"]["status"] == True:
            canBeUpdate = True
        elif len(update_info["_patches"]) > 0:
            canBeUpdate = True
        elif update_info["_deb_res"]["num"] > 0:
            canBeUpdate = True
        elif len(update_info) > 3:
            canBeUpdate = True

        if len(update_info) > 3:
            appCanBeUpdate = True
    except Exception as e:
        logging.error("refreshUpdateThread Exception:" + str(e))
        return

    # 设置checktime
    setLastCheckTime()

    if canBeUpdate:
        # if mainwindow is show
        if checkmainwindow():
            return
        if willcheckupdate:
            global n
            n = CdosNotify.Notification.new(_('CDOS Update'), _('Download and install updates?'), 'cdos-update')
            n.add_action('yes', _('yes'), enter_update_call, None, None)
            n.add_action('no', _('no'), cancel_update_call, None, None)
            n.show()
        else:
            updateapps( update_info, app_update_type_json)

        return

global bListenUpdateCache
bListenUpdateCache = False
def handle_cachestatusChanged( msg ):
    logging.info( "handle_cachestatusChanged:" + str(msg))
    global bListenUpdateCache
    global dbus_object
    if msg['status'] == 'success':
        global last_update_cache_time
        last_update_cache_time = datetime.now()
    if bListenUpdateCache:
        dbus_object.update_software_database()
        if msg['status'] == 'success':
            GLib.idle_add( refreshUpdateThread )
        if msg['status'] == 'failed':
            GLib.idle_add( refreshUpdateThread )
        bListenUpdateCache = False

def handle_statusChanged(statusCode, status_text, patchID, correct_install_patches_list, patchList, progress, paramList):
    if statusCode == PatchStatus.ALLINSTALLED:
        if ('reboot' in paramList) and (paramList['reboot'] == True):
            global n
            n = CdosNotify.Notification.new(_('CDOS Update'), _('Some updates will take effect after reboot CDOS'), 'cdos-update')
            n.show()
            pass

def refreshUpdate(isMainWindow=False):
    global bListenUpdateCache
    global dbus_object
    if checkmainwindow():
        return
    env = { 'HOME': os.environ['HOME'], 'USER': os.environ['USER'], 'DISPLAY':os.environ['DISPLAY'] }
    bListenUpdateCache = True
    dbus_object.update_cache(env)

def update_timer():
    global timer_id
    logging.debug('update_timer')

    determine_res = determineCheck()
    if timer_id > -1:
        GLib.source_remove(timer_id)
        timer_id = -1
    # 出错
    if determine_res.getError() != None:
        timer_id = GLib.timeout_add_seconds(60, update_timer)
        return Result(determine_res.getError(), None)
    # 不需要进行补丁升级检查
    if determine_res.getValue() == False:
    # if False:
        getsleeptime_res = getSleeptimeBeforeNextCheck()
        if getsleeptime_res.getError() != None:
            timer_id = GLib.timeout_add_seconds(60, update_timer)
        else:
            timer_id = GLib.timeout_add_seconds(getsleeptime_res.getValue(), update_timer)
        return Result("it's not time for patch", None)
    # 需要进行补丁升级检查
    else:
        # 检查是否需要更新
        # mainCheckWindow.set_last_checkTime(check_last_time)
        refreshUpdate()
        getsleeptime_res = getSleeptimeBeforeNextCheck()
        if getsleeptime_res.getError() != None:
            timer_id = GLib.timeout_add_seconds(60, update_timer)
        else:
            timer_id = GLib.timeout_add_seconds(getsleeptime_res.getValue(), update_timer)
    return 1

def refreshCache():
    # env = { 'HOME': os.environ['HOME'], 'USER': os.environ['USER'], 'DISPLAY':os.environ['DISPLAY'] }
    # dbus_object.update_cache(env)
    # one_hour = Timer(100, refreshCache)
    # one_hour.start()
    # return
    global last_update_cache_time
    if last_update_cache_time is None:
        if not checkmainwindow():
            env = { 'HOME': os.environ['HOME'], 'USER': os.environ['USER'], 'DISPLAY':os.environ['DISPLAY'] }
            dbus_object.update_cache(env)
        one_hour = Timer(3600, refreshCache)
        one_hour.start()
    else:
        nowtime = datetime.now()
        unchecktime = nowtime - last_update_cache_time
        one_day = timedelta(hours=24)
        nexttime = one_day - unchecktime
        if nexttime.total_seconds() > 0:
            nextcheck = Timer(nexttime.total_seconds(), refreshCache)
            nextcheck.start()
        else:
            if not checkmainwindow():
                env = { 'HOME': os.environ['HOME'], 'USER': os.environ['USER'], 'DISPLAY':os.environ['DISPLAY'] }
                dbus_object.update_cache(env)
            one_hour = Timer(3600, refreshCache)
            one_hour.start()

global client_dbus_name
global client_dbus_path
global client_dbus_iface
client_dbus_name = "org.cdos.update.client"
client_dbus_path = "/org/cdos/update/client"
client_dbus_iface =  "org.cdos.update.client"

class CDOSUpdateClientDaemon(dbus.service.Object):

    def __init__(self):
        self.session_bus = dbus.SessionBus()
        name = dbus.service.BusName(client_dbus_name, bus=self.session_bus)
        dbus.service.Object.__init__(self, name, client_dbus_path)
        self.cache=apt.Cache()

    @dbus.service.method(client_dbus_iface, in_signature='sss', out_signature='bs')
    def setAppUpdateMode(self, pkg_name, update_mode, process_name):
        self.cache.open()
        if not pkg_name in self.cache:
            return False, pkg_name + " not in cache"
        if update_mode == "AUTO_UPDATE" or update_mode == "MANUAL_UPDATE" \
            or update_mode == "AUTO_PROMPT_UPDATE" or update_mode == "DEFAULT_UPDATE":
            try:
                # schema = Gio.Settings(schema = "org.cdos.update")
                # schema.set_string("appupdatemode", update_mode)
                global app_update_type_json
                global app_update_type_db_obj
                app_name = ""
                app_update_type_db_obj.insert(pkg_name, app_name, update_mode, process_name)
                app_update_type_json[pkg_name] = {}
                app_update_type_json[pkg_name]["update_type"] = update_mode
                app_update_type_json[pkg_name]["process_name"] = process_name
                print app_update_type_json
                return True, ""
            except Exception as e:
                print str(e)
                return False, str(e)
        return False, "update mode is incorrect"

    @dbus.service.method(client_dbus_iface, in_signature='s', out_signature='s')
    def getAppUpdateMode(self, pkg_name):
        self.cache.open()
        if not pkg_name in self.cache:
            return "DEFAULT_UPDATE"
        try:
            global app_update_type_json
            if pkg_name in app_update_type_json:
                return app_update_type_json[pkg_name]["update_type"]
            else:
                return "DEFAULT_UPDATE"
        except Exception as e:
            return "DEFAULT_UPDATE"

if __name__ == "__main__":

    try:
        dbml = DBusGMainLoop(set_as_default=True)
        bus = dbus.SystemBus()
        remote_object = bus.get_object("org.cdos.update", "/org/cdos/update")
        dbus_object = dbus.Interface(remote_object, 'org.cdos.update')

        bus.add_signal_receiver(handle_cachestatusChanged,'cacheStatusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        bus.add_signal_receiver(handle_upgradeStatusChanged,'upgradeStatusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        bus.add_signal_receiver(handle_statusChanged,'statusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        dbus.set_default_main_loop(dbml)
    except Exception, e:
        logging.error("Error Happened, please check the dbus service org.cdos.update has started")
        logging.error('generic exception %s:  %s' % (e, traceback.format_exc()))
        sys.exit(1)  #进程退出

    schema = Gio.Settings(schema = "org.cdos.update")

    def on_willcheckupdate_changed(self, c):
        global willcheckupdate
        willcheckupdate = schema.get_boolean("checkupdate")

    willcheckupdate = schema.get_boolean("checkupdate")
    schema.connect("changed::checkupdate", on_willcheckupdate_changed)

    def on_dayfreq_changed(c):
        global dayfreq
        global timer_id
        dayfreq = schema.get_int("check-day")
        # mainCheckWindow.setDayFrequecy(dayfreq, hourfreq)
        if timer_id and timer_id > -1:
            GLib.source_remove(timer_id)
            timer_id = -1
        timer_id = GLib.timeout_add_seconds(0, update_timer)
        return False

    def idle_on_dayfreq_changed(self, c):
        GLib.idle_add(on_dayfreq_changed, c)

    def on_hourfreq_changed(c):
        global hourfreq
        global timer_id
        hourfreq = schema.get_int("check-hour")
        # mainCheckWindow.setDayFrequecy(dayfreq, hourfreq)
        if timer_id and timer_id > -1:
            GLib.source_remove(timer_id)
            timer_id = -1
        timer_id = GLib.timeout_add_seconds(0, update_timer)
        return False

    def idle_on_hourfreq_changed(self, c):
        GLib.idle_add(on_hourfreq_changed, c)

    dayfreq = schema.get_int("check-day")
    hourfreq = schema.get_int("check-hour")
    schema.connect("changed::check-day", idle_on_dayfreq_changed)
    schema.connect("changed::check-hour", idle_on_hourfreq_changed)

    CDOSUpdateClientDaemon()
    refreshCache()
    timer_id = GLib.timeout_add_seconds(3, update_timer)
    Gtk.main()

