#!/bin/python
#-*- coding: utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import sys
try:
    reload(sys)
    sys.setdefaultencoding('utf8')
except Exception as e:
    pass
import time
import os
import logging
import tempfile
import dbus
import dbus.service
from dbus.mainloop.glib import DBusGMainLoop


import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk,GLib,Gio,Gdk,GObject
from gi.repository.GdkPixbuf import Pixbuf,PixbufAnimation
from check_upgrade import *
from warning import *
import thread
from history import history_db

ISOTIMEFORMAT = '%Y-%m-%d %H:%M'

logging.basicConfig(format= '%(levelname)s %(asctime)-15s %(message)s file:%(filename)s line:%(lineno)d', filename=tempfile.mktemp("cdos-update-frontend.log"),level=logging.DEBUG)

class Result:
    __error = None
    __value = None

    def __init__(self, error, value):
        self.__error = error
        self.__value = value

    def getError(self):
        return self.__error

    def getValue(self):
        return self.__value

class PatchStatus:
    INITED = 0
    DOWNLOADING = 10
    DOWNLOADED = 14
    ALLDOWNLOADED = 15
    UPDATING = 20
    VERIFYING = 22
    INSTALLING = 23
    INSTALLED = 25
    ALLINSTALLED = 30
    ERROR_INSTALL = 51

class CheckMainwindow(object):
    
    def __init__(self, img_dir ,uidir):
        import gettext as gettext1
        gettext1.bindtextdomain('cdosupdate', "/usr/share/locale/")
        gettext1.textdomain('cdosupdate')
        self._ = gettext1.gettext

        provider = Gtk.CssProvider()
        provider.load_from_data(b'''\

            #note-viewport {
                background:#FCFCFC;
            }

            #main_notebook {
                border: none;
            }
            #setting_button,
            #detail_button {
                border: none;
                color: #007dce;
                font-size: 12px;
                background: transparent;
            }
            #detail_button:active{
                border: none;
            }
            #update_name {
                font-size: 18px;
                color: #333;
            }
            #update_name_msg,
            #detail_label {
                font-size: 12px;
                color: #888;
            }
            #update_button {
                font-size: 14px;
                color: #333;
            }

            #update_button:insensitive {
                background: #cccccc;
                color: #FFFFFF;
                border: none;
            }

            #update_button,
            #loadingfailed_button,
            #enterbutton,
            #enterbutton1,
            #confirm {
                -GtkWidget-focus-padding: 0;
                -GtkWidget-focus-line-width: 0;
                -GtkButton-inner-border: 0;

                background: #44A6E5;
                color: #FFFFFF;
                box-shadow: none;
                font-weight: normal;
                font-size: 13px;
                border: none;
                border-radius: 0;
                border-color: #44A6E4;
                border-width: 1px;
                border-style: solid;
            }

            #update_button:active,
            #update_button:active:hover,
            #loadingfailed_button:active,
            #loadingfailed_button:active:hover,
            #enterbutton:active,
            #enterbutton:active:hover,
            #enterbutton1:active,
            #enterbutton1:active:hover,
            #confirm:active,
            #confirm:active:hover {
                background: #3C93D3;
                color: #FFFFFF;
                box-shadow: 0px 0px 0px 0px;
            }

            #update_button:hover,
            #loadingfailed_button:hover,
            #enterbutton:hover,
            #enterbutton1:hover,
            #confirm:hover {
                background: #48B2ED;
                color: #FFFFFF;
                box-shadow: 0px 0px 0px 0px;
            }

            #update_button:focus,
            #update_button:hover:focus,
            #update_button:active:focus,
            #loadingfailed_button:focus,
            #loadingfailed_button:hover:focus,
            #loadingfailed_button:active:focus,
            #enterbutton:focus,
            #enterbutton:hover:focus,
            #enterbutton:active:focus,
            #enterbutton1:focus,
            #enterbutton1:hover:focus,
            #enterbutton1:active:focus,
            #confirm:focus,
            #confirm:hover:focus,
            #confirm:active:focus,
            #cancel:focus,
            #cancel:hover:focus,
            #cancel:active:focus {
                border-color: shade(#44A6E4, 0.8);
                box-shadow: 0px 0px 0px 0px;
            }

            #confirm:insensitive,
            #confirm :insensitive,
            #cancel:insensitive,
            #cancel :insensitive {
                background: #cccccc;
                color: #FFFFFF;
                border: none;
            }

            #loadingfailed_label {
                color: #888888;
            }
            #loadingfailed_detail_label {
                color: #999999;
            }

            ''')
        Gtk.StyleContext.add_provider_for_screen(
            Gdk.Screen.get_default(),
            provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)

        self.builder = Gtk.Builder()
        self.builder.add_from_file(os.path.join(uidir, "mainwindow3.ui"))
        self.builder.connect_signals(self)
        self.window = self.builder.get_object("mainwindow")
        self.window.set_title(self._("CDOS Update"))
        self.window.set_size_request(960, 540)

        self.note_viewport = self.builder.get_object("note-viewport")
        self.note_viewport.set_name("note-viewport")
        self.notepage = self.builder.get_object("notebook_page")
        self.notepage.set_name("notebook_page")
        self.progress_pages = {
            'main': 0,
            'settings': 1,
        }

        self.img_dir = img_dir
        self.uidir = uidir
        self.app_num = 0
        # update info
        self.updates_handler_json = {
            "upgrade": None,
            "system": None,
            "apps": {}
        }
        self.app_query = []

        self.current_work_type = ""
        self.current_work = {}
        self.busy_flag = False
        self.upgrade_flag = False
        self.env = {}
        self.env["HOME"] = os.getenv("HOME")
        self.env["USER"] = os.getenv("USER")
        self.env["DISPLAY"] = os.getenv("DISPLAY")
        # query need to be exec
        self.query = []
        self.current_obj = None

        # init url for updates
        self.init_updates_url()
        self._init_dbus_daemon_handler()

        self.init_notebook_pages()
        try:
            home_dir = os.getenv("HOME")
            self.history_db = history_db(os.path.join(home_dir, ".config/cdos-update/history.db"))
        except Exception as e:
            self.history_db = None

        self.window.connect("delete-event", self.quit_window)

    def quit_window(self, widget, event):
        if self.busy_flag == False:
            self.do_close()
        else:
            warning_dialog = WarningDialog(self, self.uidir, self.img_dir)
            warning_dialog.set_label(self._("CDOS Update is working.\nClose it later."))
            warning_dialog.show()
            return True

    def do_close(self):
        Gtk.main_quit()

    def init_updates_url(self):
        self.repo_url = "http://repos.01os.org/"
        self.site_path = self.repo_url + "data/cdos-update/update/"
        self.getpatch_site_path = self.site_path + "getpatch/"
        self.getpatchlist_site_path = self.site_path + "getpatchlist"
        self.getpatchlistinfo_site_patch = self.site_path + "getpatchesinfoWithSystemVersion"
        self.upgrade_path = self.repo_url + "data/cdos-update/cdos-upgrade-new-server/meta-release"

    def _init_dbus_daemon_handler(self):
        intern_dbus_name = "org.cdos.update" 
        intern_dbus_path = "/org/cdos/update"
        intern_dbus_iface = "org.cdos.update"
        self.system_bus = dbus.SystemBus()
        self.remote_obj = self.system_bus.get_object( intern_dbus_name, intern_dbus_path )
        self.iface = dbus.Interface( self.remote_obj, intern_dbus_iface)
        self.system_bus.add_signal_receiver(self.handle_statusChanged,'statusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        self.system_bus.add_signal_receiver(self.handle_aptStatusChanged,'AptStatusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        self.system_bus.add_signal_receiver(self.handle_upgradeStatusChanged,'upgradeStatusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")

    def commit_system_update(self, getpatch_info_site_path, getpatch_site_path, pkgs_install=[], pkgs_upgrade=[]):
        self.iface.commit_system_updates( self.env, pkgs_install, pkgs_upgrade, getpatch_info_site_path, getpatch_site_path)

    def commit_app_update(self, pkgs_install=[], pkgs_upgrade=[]):
        self.iface.commit_app_pkgs( self.env, pkgs_install, pkgs_upgrade)

    def install_patches(self, getpatchlistinfo_site_patch, getpatch_site_path):
        self.iface.install( getpatch_site_path, self.iface.get_patchlist_to_install( getpatchlistinfo_site_patch))
        return

    def get_patches_to_install(self, getpatchlistinfo_site_patch):
        return self.iface.get_patchlist_to_install( getpatchlistinfo_site_patch)

    def exec_query(self):
        if self.busy_flag:
            return
        else:
            self.busy_flag = True
            self.do_exec_query()

    def do_exec_query(self):
        if len(self.query) > 0:
            work = self.query.pop(0)
            if work["type"] == "system_update":
                self.current_work_type = "system_update"
                self.current_work = work["ev_info"]
                self.current_obj = self.updates_handler_json["system"]
                self.commit_system_update( getpatch_info_site_path=self.getpatchlistinfo_site_patch, getpatch_site_path=self.getpatch_site_path, pkgs_install=list(work["ev_info"]["debs"]))
            elif work["type"] == "app_update":
                self.current_work_type = "app_update"
                self.current_work = work["ev_info"]
                self.current_obj = self.updates_handler_json["apps"][work["ev_info"]["debs"][0]]
                # install app deb
                self.commit_app_update( pkgs_install=list(work["ev_info"]["debs"]))

    def push_query(self, ev):
        self.query.append(ev)
        self.exec_query()

    def pop_query(self, ev):
        self.query.remove(ev)

    def handle_upgradeStatusChanged(self, statusCode, status_text, paramList):
        def dist_upgrade(upgrade_cmd):
            logging.info( upgrade_cmd)
            os.system("bash " + upgrade_cmd + " show")

        self.updates_handler_json["upgrade"].show_progress_box()
        self.updates_handler_json["upgrade"].progressbar_info_label.set_text(self._(status_text))
        if statusCode == 101:
            self.upgrade_flag = False
            self.set_update_sensitive(True)
            logging.info( "statusCode is 101")
            self.updates_handler_json["upgrade"].show_button()
            # ToDo: restore status
            return
        self.updates_handler_json["upgrade"].progressbar.set_fraction(statusCode/100.0)
        if statusCode == 100:
            try:
                upgrade_tool_main_path = os.path.join(paramList["upgrade_tool_dir"], "distupgrade")
                thread.start_new_thread(dist_upgrade, (upgrade_tool_main_path, ))
                time.sleep(1)
            except Exception as e:
                logging.error("handle_upgradeStatusChanged:" + str(e))
            finally:
                self.do_close()
                pass

    def handle_statusChanged(self, statusCode, status_text, patchID, correct_install_patches_list, patchList, progress, paramList):
        current_obj = self.updates_handler_json["system"]
        if statusCode == PatchStatus.ERROR_INSTALL:
            warning_dialog = WarningDialog(self, self.uidir, self.img_dir, True)
            warning_dialog.set_label(self._("Error occurs when patches installed.\nClose the window."))
            warning_dialog.show()
            return
        if statusCode == PatchStatus.ALLINSTALLED:
            current_obj.set_progress_bar(1.0)
            current_obj.finish()
            # history system
            update_length = len(current_obj.deb_list)+len(correct_install_patches_list)
            if update_length == 1:
                short_desc = self._("Install %d update") %update_length
            else:
                short_desc = self._("Install %d updates") %update_length
            long_desc = ""
            if not current_obj.history_long_desc == "":
                long_desc = current_obj.history_long_desc + "\n"
            if len(correct_install_patches_list) == 0:
                pass
            elif len(correct_install_patches_list) == 1:
                long_desc += self._("Install %d patch:") %len(correct_install_patches_list)
            else:
                long_desc += self._("Install %d patches:") %len(correct_install_patches_list)
            for patch in correct_install_patches_list:
                long_desc += " " + str(patch)
            try:
                self.history_db.insert(update_type="systemupdate", updateTime=int(time.time()), appname=current_obj.update_name, version="", size=current_obj.size, icon=current_obj.image_name, shortdesc=short_desc, longdesc=long_desc)
                self.inster_history(update_type="update", image_name=current_obj.image_name, update_name=current_obj.update_name, update_name_msg=short_desc, show_detail=True, detail_info=long_desc)
            except Exception as e:
                logging.info("insert history Exception")
            if len(self.query) > 0:
                self.do_exec_query()
            else:
                self.busy_flag = False
        else:
            current_obj.set_progress_bar(80.0 + float(progress)*0.2)

    def handle_aptStatusChanged(self, status):
        if status["type"] == "app_update":
            if status["first_deb"] in self.updates_handler_json["apps"]:
                current_obj = self.updates_handler_json["apps"][status["first_deb"]]
        elif status["type"] == "system_update":
            current_obj = self.updates_handler_json["system"]
        else:
            return
        current_obj.hide_button()

        if status["status"] == "inprogress":
            if status["type"] == "system_update":
                current_obj.set_progress_bar(status["percent"]*0.8)
            else:
                current_obj.set_progress_bar(status["percent"])
        elif status["status"] == "success" and status["type"] == "app_update":
            current_obj.finish()

            self.app_num = self.app_num - 1
            if self.app_num <= 0:
                self.app_title_grid.hide()
            elif self.app_num == 1:
                self.app_label.set_text(self._("%d application to be updated") %self.app_num)
            else:
                self.app_label.set_text(self._("%d applications to be updated") %self.app_num)
            # history app
            try:
                self.history_db.insert(update_type="application", updateTime=int(time.time()), appname=current_obj.update_name, version=current_obj.version, size=current_obj.size, icon=current_obj.image_name, shortdesc=current_obj.update_name_msg, longdesc=current_obj.detail_info)
                self.inster_history(update_type="app", image_name=current_obj.image_name, update_name=current_obj.update_name, update_name_msg=current_obj.update_name_msg, show_detail=True, detail_info=current_obj.history_long_desc)
            except Exception as e:
                logging.info("insert history Exception")
            if len(self.query) > 0:
                self.do_exec_query()
            else:
                self.busy_flag = False
        elif status["status"] == "failed":
            if status['error'] == 'download failed':
                self.on_download_failed( current_obj)
                return
            current_obj.finish_exec()
            error_msg = self._("Error occurs when debs installed.\nRun \"sudo apt-get install -f\" to fix it.")
            warning_dialog = WarningDialog(self, self.uidir, self.img_dir)
            warning_dialog.set_label(error_msg)
            warning_dialog.show()
            if len(self.query) > 0:
                self.do_exec_query()
            else:
                self.busy_flag = False
            return

    def on_download_failed(self, current_obj):
        current_obj.finish_exec()
        self.current_work_type = ""
        self.busy_flag = False
        while len(self.query) > 0:
            work = self.query.pop(0)
            one_obj = None
            if work["type"] == "system_update":
                one_obj = self.updates_handler_json["system"]
            elif work["type"] == "app_update":
                one_obj = self.updates_handler_json["apps"][work["ev_info"]["debs"][0]]
            if one_obj is not None:
                one_obj.finish_exec()
        error_msg = self._('Download failed, please check the network connection.')
        warning_dialog = WarningDialog(self, self.uidir, self.img_dir)
        warning_dialog.set_label(error_msg)
        warning_dialog.show()

    def get_dbus_daemon(self):
        return self.iface

    def get_updates_info(self):
        try:
            res = {}
            res = self.iface.getAllUpdates( os.environ["HOME"] ,self.upgrade_path, self.getpatchlistinfo_site_patch)
            return Result(None, res)
        except Exception as e:
            logging.info("get_updates_info Exception:" + str(e))
            return Result("get_updates_info error:" + str(e), None)

    def show(self):
        self.window.show()

    # main page
    def go_to_main_page(self):
        self.notepage.set_current_page(self.progress_pages['main'])

    def check_network(self):
        # 无法检测404异常，仅用来检测是否有网络链接
        res = None
        try:
            res = urllib.urlopen(self.repo_url)
        except Exception as e:
            logging.error(str(e))
            return False
        content = str(res.read())
        if content == 'fail':
            return False
        return True

    # main page -> sucess page
    def go_to_updates_page(self):

        def load_updates_info(self):
            if not self.check_network():
                time.sleep(1)
                self.go_to_loadingfailed_page()
                return
            updates_info = self.get_updates_info()
            # error or no value
            if (updates_info.getError() is not None) or (updates_info.getValue() is None):
                self.go_to_loadingfailed_page()
                return

            updates_info_value = updates_info.getValue()
            def dist_upgrade(upgrade_cmd):
                logging.info("upgrade_cmd:" + upgrade_cmd)
                os.system("bash " + upgrade_cmd + " show")
            try:
                # if is downloading/upgrading
                if ( "_upgrade" in updates_info_value) and ("upgrading" in updates_info_value["_upgrade"]) and (updates_info_value["_upgrade"]["upgrading"]) and ( "upgrade_tool_dir" in updates_info_value["_upgrade"]) and ( not updates_info_value["_upgrade"]["upgrade_tool_dir"] == ""):
                    try:
                        upgrade_tool_main_path = os.path.join( updates_info_value["_upgrade"]["upgrade_tool_dir"], "distupgrade")
                        thread.start_new_thread(dist_upgrade, (upgrade_tool_main_path, ))
                        time.sleep(1)
                    except Exception as e:
                        logging.error("load_updates_info2:" + str(e))
                    finally:
                        self.do_close()
            except Exception as e:
                pass

            GLib.idle_add(self.go_to_loadingsucceed_page, updates_info.getValue())
        thread.start_new_thread(load_updates_info, (self,))

    # setting page
    def go_to_settings_page(self):
        self.notepage.set_current_page(self.progress_pages['settings'])

    def init_notebook_pages(self):
        self.init_main_page()
        self.init_setting_page()

    def init_main_page(self):
        self.main_notebook = self.builder.get_object("main_notebook")
        self.main_notebook.set_name("main_notebook")
        self.main_notebook_pages = {
            'loading': 0,
            'loadingfailed': 1,
            'loadingsucceed': 2,
        }

        self.setting_button = self.builder.get_object("setting_button")
        self.setting_button.set_name("setting_button")
        self.setting_button.set_label(self._("settings..."))
        self.top_label = self.builder.get_object("top_label")
        self.top_label.set_name("top_label")

        self.loading_label = self.builder.get_object("loading_label")
        self.loading_label.set_name("loading_label")
        self.loading_label.set_text(self._("Checking updates..."))

        self.loading_image = self.builder.get_object("loading_image")
        self.loading_image.set_name("loading_image")
        loading_image_wigth = 20
        loading_image_height = 20
        # loading_image_name = "/usr/share/cdos-update/frontend/img/loading.gif"
        loading_image_name = os.path.join( self.img_dir, "loading.gif")
        loading_pixbuf = PixbufAnimation.new_from_file( loading_image_name)
        self.loading_image.set_from_animation(loading_pixbuf)

        self.loadingfailed_label = self.builder.get_object("loadingfailed_label")
        self.loadingfailed_label.set_name("loadingfailed_label")
        self.loadingfailed_label.set_text(self._("Check updates failed!"))
        self.loadingfailed_button = self.builder.get_object("loadingfailed_button")
        self.loadingfailed_button.set_name("loadingfailed_button")
        self.loadingfailed_button.set_label(self._("Check again"))
        self.loadingfailed_button.connect('clicked', self.on_reloading)

        self.loadingfailed_detail_label = self.builder.get_object("loadingfailed_detail_label")
        self.loadingfailed_detail_label.set_name("loadingfailed_detail_label")
        failed_datail_info = self._("Possible reasons:")
        failed_datail_info += "\n" + "1." + self._("Network connection exception, pelase check it.")
        failed_datail_info += "\n" + "2." + self._("Exception for deb dependency, you can run \"sudo apt-get install -f\" to fix it.")
        failed_datail_info += "\n" + "3." + self._("Unknown exceptions, try it again.")
        self.loadingfailed_detail_label.set_text(failed_datail_info)

    def on_reloading(self, widget):
        self.go_to_loading_page()
        self.go_to_updates_page()

    # loading
    def go_to_loading_page(self):
        self.main_notebook.set_current_page(self.main_notebook_pages['loading'])
        self.notepage.set_current_page(self.progress_pages['main'])
        self.top_label.hide()

    # loading failed
    def go_to_loadingfailed_page(self):
        self.main_notebook.set_current_page(self.main_notebook_pages['loadingfailed'])
        self.notepage.set_current_page(self.progress_pages['main'])
        self.top_label.hide()

    # load sucess
    def go_to_loadingsucceed_page(self, info={}):
        self.top_label.set_text(self._("System Update"))
        self.top_label.show()
        self.loading_succeed_page(info=info)

        self.main_notebook.set_current_page(self.main_notebook_pages['loadingsucceed'])
        self.notepage.set_current_page(self.progress_pages['main'])

    def set_update_sensitive(self, status):
        try:
            self.system_update_obj.button.set_sensitive(status)
        except Exception as e:
            pass
        finally:
            pass
        try:
            self.app_button.set_sensitive(status)
        except Exception as e:
            pass
        finally:
            pass
        try:
            for tmp_app_obj in self.app_query:
                tmp_app_obj.button.set_sensitive(status)
        except Exception as e:
            pass
        finally:
            pass

    def loading_succeed_page(self, info={}):
        self.loadingsucceed_box = self.builder.get_object("loadingsucceed_box")
        self.loadingsucceed_box.set_name("loadingsucceed_box")
        self.loadingsucceed_box.set_halign(Gtk.Align.FILL)
        self.loadingsucceed_box.set_valign(Gtk.Align.START)
        self.loadingsucceed_grid = Gtk.Grid()

        i = 0
        separator = Gtk.HSeparator()
        separator.show()
        self.loadingsucceed_box.pack_end(self.loadingsucceed_grid, False, False, 0)
        self.loadingsucceed_grid.attach(separator, 0, i, 3, 1)
        i += 1

        # system upgrade
        upgrade_image_name = os.path.join( self.img_dir, "cdos.png")
        if ( "_upgrade" in info) and ( "new_ver" in info["_upgrade"]) and ( "new_release" in info["_upgrade"]) and ( "status" in info["_upgrade"]) and (info["_upgrade"]["status"]):
            upgrade_name = self._("CDOS Upgrade to %s") %str(info["_upgrade"]["new_ver"])
            upgrade_name_msg = str(info["_upgrade"]["new_ver"])
            detail_info = self._("Your CDOS can be upgraded to %s") %str(info["_upgrade"]["new_ver"])
            def upgrade_event_callback():
                if self.busy_flag == True:
                    warning_dialog = WarningDialog(self, self.uidir, self.img_dir)
                    warning_dialog.set_label(self._("CDOS Update is working.\nTry it later."))
                    warning_dialog.show()
                    return
                # ToDo: do upgrade
                self.upgrade_flag = True
                self.set_update_sensitive(False)
                self.iface.can_dist_upgrade()
                self.updates_handler_json["upgrade"].hide_button()
                self.updates_handler_json["upgrade"].show_progress_box()
                self.updates_handler_json["upgrade"].progressbar.set_fraction(0)
                self.updates_handler_json["upgrade"].progressbar_info_label.set_text(self._("Prepareing upgrade tools"))
        else:
            upgrade_name = self._("Your CDOS is the newest version")
            upgrade_name_msg = str(info["_upgrade"]["new_release"])
            detail_info = self._("CDOS version is: %s") %str(info["_upgrade"]["new_release"])
            upgrade_event_callback = None
        show_upgrade_detail = True
        system_upgrade_obj = Updates_Object(update_type="upgrade" ,image_name=upgrade_image_name, update_name=upgrade_name, update_name_msg=upgrade_name_msg, 
            show_detail=show_upgrade_detail, detail_info=detail_info, button_callback=upgrade_event_callback)
        self.updates_handler_json["upgrade"] = system_upgrade_obj
        self.loadingsucceed_grid.attach(system_upgrade_obj, 0, i, 3, 1)
        i += 1
        # if you have system updates
        if ("_deb_res" in info) and ("_patches" in info) and ("num" in info["_deb_res"]) and (len(info["_patches"])+info["_deb_res"]["num"] > 0):
            update_image_name = os.path.join( self.img_dir, "cdos_core.png")
            sys_up_name = self._("CDOS core updates")
            if len(info["_patches"])+info["_deb_res"]["num"] == 1:
                sys_up_msg = self._("You have %d update") %(len(info["_patches"])+info["_deb_res"]["num"])
            else:
                sys_up_msg = self._("You have %d updates") %(len(info["_patches"])+info["_deb_res"]["num"])
            detail_info = ""
            history_long_desc = ""
            if info["_deb_res"]["num"] > 0:
                if info["_deb_res"]["num"] == 1:
                    detail_info += self._("You have %d core deb, %s") %(info["_deb_res"]["num"], str(info["_deb_res"]["size"]))
                    history_long_desc = self._("Install %d core deb, %s") %(info["_deb_res"]["num"], str(info["_deb_res"]["size"]))
                else:
                    detail_info += self._("You have %d core debs, %s") %(info["_deb_res"]["num"], str(info["_deb_res"]["size"]))
                    history_long_desc = self._("Install %d core debs, %s") %(info["_deb_res"]["num"], str(info["_deb_res"]["size"]))
            else:
                info["_deb_res"]["list"] = []
            if len(info["_patches"]) > 0:  # if patches
                if info["_deb_res"]["num"] > 0:  # if patches and debs
                    detail_info +=  "\n"
                if len(info["_patches"]) == 1:
                    detail_info +=  self._("%d patch may be installed") %len(info["_patches"])
                else:
                    detail_info +=  self._("%d patches may be installed") %len(info["_patches"])
            for patch in info["_patches"]:
                detail_info +=  "\n" + "  " + str(patch) + ": " + str(info["_patches"][patch]["description"])
            def update_event_callback(ev_type, deb_list, obj, action):
                ev = {
                    "type":"system_update",
                    "ev_info": {
                        "debs": deb_list
                    }
                }
                if action == "update":
                    GLib.idle_add(self.push_query, ev)
                else:
                    GLib.idle_add(self.pop_query, ev)

            self.system_update_obj = Updates_Object(update_type="update" ,image_name=update_image_name, update_name=sys_up_name, update_name_msg=sys_up_msg, 
                show_detail=True, detail_info=detail_info, size=info["_deb_res"]["size"], button_callback=update_event_callback, deb_list=info["_deb_res"]["list"], history_long_desc=history_long_desc)
            self.updates_handler_json["system"] = self.system_update_obj
            self.loadingsucceed_grid.attach(self.system_update_obj, 0, i, 3, 1)
            i += 1

        if len(info) > 3:
            self.app_title_grid = Gtk.Grid()
            self.app_label = Gtk.Label()
            self.app_label.set_xalign(0)
            alignment = Gtk.Alignment()
            alignment.set_hexpand(True)
            alignment.set_halign(Gtk.Align.FILL)
            self.app_button = Gtk.LinkButton()
            self.app_button.set_label(self._("update all applications"))
            self.app_title_grid.attach(self.app_label, 0, 0, 1, 1)
            self.app_title_grid.attach(alignment, 1, 0, 1, 1)
            self.app_title_grid.attach(self.app_button, 2, 0, 1, 1)
            self.app_label.set_margin_top(34)
            self.app_label.set_name("top_label")
            self.app_button.set_margin_top(34)
            self.app_button.set_name("detail_button")
            self.app_label.show()
            alignment.show()
            self.app_button.show()
            app_separator = Gtk.HSeparator()
            app_separator.show()
            self.app_title_grid.attach(app_separator, 0, 1, 3, 1)
            self.loadingsucceed_grid.attach(self.app_title_grid, 0, i, 3, 1)
            i += 1

            for app_info in info:
                if ((app_info == "_deb_res") or (app_info == "_patches") or (app_info == "_upgrade")):
                    continue
                try:
                    app_name = info[app_info]["AppName"]
                    app_msg = self._("Version:")  + info[app_info]["Version"]
                    app_details = info[app_info]["LongDesc"] + "\n" + self._("Version:")  + info[app_info]["Version"] + "\n" + self._("Size:") + info[app_info]["size"]
                    icon = info[app_info]["Icon"]
                    def app_update_event_callback(ev_type, deb_list, obj, action):
                        ev = {
                            "type":"app_update",
                            "ev_info": {
                                "debs": deb_list
                            }
                        }
                        if action == "update":
                            GLib.idle_add(self.push_query, ev)
                        else:
                            GLib.idle_add(self.pop_query, ev)

                    app_obj = Updates_Object(update_type="app", image_name=icon, update_name=app_name, update_name_msg=app_msg, 
                        show_detail=True, detail_info=app_details, version=info[app_info]["Version"] ,size=info[app_info]["size"] ,button_callback=app_update_event_callback, deb_list=[info[app_info]["Package"], ])
                except Exception as e:
                    continue
                self.updates_handler_json["apps"][app_info] = app_obj
                self.app_query.append(app_obj)
                self.loadingsucceed_grid.attach(app_obj, 0, i, 3, 1)
                self.app_num = len(self.updates_handler_json["apps"])
                if self.app_num == 1:
                    self.app_label.set_text(self._("%d application to be updated") %self.app_num)
                else:
                    self.app_label.set_text(self._("%d applications to be updated") %self.app_num)
                if self.app_num > 0:
                    self.app_title_grid.show()

                app_obj.show()
                i += 1
            def on_app_button_clicked(tmp_arg):
                for tmp_app_obj in self.app_query:
                    if tmp_app_obj.ready == False:
                        tmp_app_obj.do_work()
            self.app_button.connect("clicked", on_app_button_clicked)

        if self.history_db is None:
            self.loadingsucceed_grid.show()
            return

        self.history_title_grid = Gtk.Grid()
        self.history_label = Gtk.Label()
        self.history_label.set_text(self._("history in 30 days"))
        self.history_label.set_xalign(0)
        self.history_label.set_margin_end(8)
        self.alignment = Gtk.Alignment()
        self.alignment.set_hexpand(True)
        self.alignment.set_halign(Gtk.Align.FILL)
        self.alignment.set_margin_end(8)
        self.history_title_grid.attach(self.history_label, 0, 0, 1, 1)
        self.history_title_grid.attach(self.alignment, 1, 0, 1, 1)
        self.history_label.set_margin_top(34)
        self.history_label.set_name("top_label")
        self.history_separator = Gtk.HSeparator()
        self.alignment.set_margin_top(8)
        self.history_title_grid.attach(self.history_separator, 0, 1, 3, 1)
        self.loadingsucceed_grid.attach(self.history_title_grid, 0, i, 3, 1)
        i += 1
        self.history_inster = i
        try:
            history_infos = self.history_db.search(30)
            self.his_length = len(history_infos)

            for history_info in history_infos:
                try:
                    update_type = history_info["type"]
                    app_name = history_info["appname"]
                    version = history_info["version"]
                    size = history_info["size"]
                    icon = history_info["icon"]
                    shortdesc = history_info["shortdesc"]
                    longdesc = history_info["longdesc"]
                    if update_type == "systemupdate":
                        app_obj = Updates_Object(update_type="update", image_name=icon, update_name=app_name, update_name_msg=shortdesc, 
                            show_detail=True, detail_info=longdesc)
                    elif update_type == "application":
                        app_obj = Updates_Object(update_type="app", image_name=icon, update_name=app_name, update_name_msg=shortdesc, 
                            show_detail=True, detail_info=longdesc)
                except Exception as e:
                    logging.info("Insert history Exception:" + str(e))
                    continue
                self.loadingsucceed_grid.insert_row(i)
                self.loadingsucceed_grid.attach(app_obj, 0, i, 3, 1)
                app_obj.show()

        except Exception as e:
            logging.info("Insert history Exception:" + str(e))
            self.his_length = 0

        self.show_history_title()

        self.loadingsucceed_grid.show()

    def inster_history(self, update_type, image_name, update_name, update_name_msg, show_detail, detail_info):
        try:
            app_obj = Updates_Object(update_type=update_type, image_name=image_name, update_name=update_name, update_name_msg=update_name_msg, 
                show_detail=show_detail, detail_info=detail_info)
            self.loadingsucceed_grid.insert_row(self.history_inster)
            self.loadingsucceed_grid.attach(app_obj, 0, self.history_inster, 3, 1)
            app_obj.show()
            self.his_length += 1
            self.show_history_title()
        except Exception as e:
            pass

    def show_history_title(self):
        if self.his_length > 0:
            self.history_label.show()
            self.alignment.show()
            self.history_separator.show()
            self.history_title_grid.show()
        else:
            self.history_label.hide()
            self.alignment.hide()
            self.history_separator.hide()
            self.history_title_grid.hide()

    def on_setting_button_clicked(self, widget):
        self.go_to_settings_page()

    def init_setting_page(self):
        self.setting_label = self.builder.get_object("setting_label")
        self.setting_label.set_name("setting_label")
        self.setting_label.set_text(self._("Choose a method to install updates CDOS"))
        self.setting_label2 = self.builder.get_object("setting_label2")
        self.setting_label2.set_name("setting_label2")
        self.setting_label2.set_text(self._("When the computer is online, CDOS can automatically check for critical updates and remind you according to your setup."))
        self.time_setting_label = self.builder.get_object("time_setting_label")
        self.time_setting_label.set_name("time_setting_label")
        self.time_setting_label.set_text(self._("Install the new update:"))
        self.time_setting_label2 = self.builder.get_object("time_setting_label2")
        self.time_setting_label2.set_name("time_setting_label2")
        self.time_setting_label2.set_text(self._("at"))
        # setting frequency
        self.updatefrequence_viewport = self.builder.get_object("updatefrequence_viewport")
        self.updatefrequence_viewport.set_name("updatefrequence_viewport")
        self.dayfreq_viewport = self.builder.get_object("dayfreq_viewport")
        self.dayfreq_viewport.set_name("dayfreq_viewport")
        self.hourfreq_viewport = self.builder.get_object("hourfreq_viewport")
        self.hourfreq_viewport.set_name("hourfreq_viewport")

        self.updateFrequence_list_store = Gtk.ListStore(GObject.TYPE_STRING)
        self.updateFrequenceArray = [self._("Check for updates and remind me"), self._("Never check for updates (not recommended)")]
        for item in self.updateFrequenceArray:
            self.updateFrequence_list_store.append([item])
        self.updatefrequence = self.builder.get_object("updatefrequence")
        self.updatefrequence.set_name('updatefrequence')
        self.updatefrequence.set_model(self.updateFrequence_list_store)
        self.updatefrequence.set_active(0)
        self.updatefrequence.connect("changed", self.on_changed_updatefrequence)

        self.dayfreq_list_store = Gtk.ListStore(GObject.TYPE_STRING)
        self.dayfreqArray = [self._("Everyday"), self._("Every Monday"), self._("Every Tuesday"), self._("Every Wednesday"), self._("Every Thursday"), self._("Every Friday"), self._("Every Saturday"), self._("Every Sunday")]
        for item in self.dayfreqArray:
            self.dayfreq_list_store.append([item])
        self.dayfreq = self.builder.get_object("dayfreq")
        self.dayfreq.set_name('dayfreq')
        self.dayfreq.set_model(self.dayfreq_list_store)
        self.dayfreq.set_active(0)
        self.dayfreq.connect("changed", self.on_changed_dayfreq)

        self.hourfreq_list_store = Gtk.ListStore(GObject.TYPE_STRING)
        for i in range(24):
            self.hourfreq_list_store.append([str(i)+":00"])
        self.hourfreq = self.builder.get_object("hourfreq")
        self.hourfreq.set_name('hourfreq')
        self.hourfreq.set_model(self.hourfreq_list_store)
        self.hourfreq.set_active(0)
        self.hourfreq.connect("changed", self.on_changed_hourfreq)
        # dist-upgrade options
        self.distupgrade_label = self.builder.get_object("distupgrade_label")
        self.distupgrade_label.set_name("distupgrade_label")
        self.distupgrade_label.set_text(self._("System dist-upgrade options:"))
        self.distupgrade_list_store = Gtk.ListStore(GObject.TYPE_STRING)
        self.distupgrade_list_array = [self._("Auto dist-upgrade"), self._("Remind me after downloaded")]
        for item in self.distupgrade_list_array:
            self.distupgrade_list_store.append([item])
        self.distupgrade_list = self.builder.get_object("distupgrade_list")
        self.distupgrade_list.set_name('distupgrade_list')
        self.distupgrade_list.set_model(self.distupgrade_list_store)
        self.distupgrade_list.set_active(0)
        self.distupgrade_list.connect('changed', self.on_changed_distupgrade)
        self.distupgrade_note = self.builder.get_object("distupgrade_note")
        self.distupgrade_note.set_name("distupgrade_note")
        self.distupgrade_note.set_text(self._("note: Checking upgrade is as the same time checking update"))
        # confirm or cancel
        self.confirm_btn = self.builder.get_object('confirm')
        self.confirm_btn.set_name('confirm')
        self.confirm_btn.set_label(self._('Confirm'))
        self.confirm_btn.connect("button_press_event", self.on_confirm_press)
        self.confirm_btn.connect("clicked", self.on_confirm_click)
        self.cancel_btn = self.builder.get_object('cancel')
        self.cancel_btn.set_name('cancel')
        self.cancel_btn.set_label(self._('Cancel'))
        self.cancel_btn.connect("clicked", self.on_cancel_click)
        self.confirm_btn.set_sensitive(False)

        self.__get_gio_setting()

    def __get_gio_setting(self):
        schema = Gio.Settings(schema = "org.cdos.update")
        dayfreq = schema.get_int("check-day")
        hourfreq = schema.get_int("check-hour")
        self.setDayFrequecy(dayfreq, hourfreq)
        autoinstall = schema.get_boolean("autoinstall")
        autodownload = schema.get_boolean("autodownload")
        willcheckupdate = schema.get_boolean("checkupdate")
        distupgrade_opts = schema.get_int("distupgrade-opts")
        self.setFrequecy(willcheckupdate, autodownload, autoinstall)
        self.set_distupgrade(distupgrade_opts)
        self.confirm_btn.set_sensitive(False)
        self.cancel_btn.set_sensitive(True)

    def setFrequecy(self, willcheckupdate, autodownload, autoinstall):
        frequecy = None
        if not willcheckupdate:
            frequecy = self.updateFrequenceArray[1]
        else:
            frequecy = self.updateFrequenceArray[0]

        iterator = self.updateFrequence_list_store.get_iter_first()
        while iterator:
            if self.updateFrequence_list_store[iterator][0] == frequecy:
                if iterator != self.updatefrequence.get_active_iter():
                    self.updatefrequence.set_active_iter(iterator)
                break
            iterator = self.updateFrequence_list_store.iter_next(iterator)
        self.set_updatefrequence()

    # def on_changed_updatefrequence(self, combobox):
    def set_updatefrequence(self):
        combobox = self.updatefrequence
        model = combobox.get_model()
        iterator = combobox.get_active_iter()
        chosen_item = -1
        for i in range(len(self.updateFrequenceArray)):
            if self.updateFrequenceArray[i] == model[iterator][0]:
                chosen_item = i
                break
        willcheckupdate = True
        if chosen_item >= 0:
            if chosen_item == 0:
                willcheckupdate = True
                if not self.dayfreq.is_sensitive():
                    self.dayfreq.set_sensitive(True)
                    self.hourfreq.set_sensitive(True)
            elif chosen_item == 1:
                willcheckupdate = False
                if self.dayfreq.is_sensitive():
                    self.dayfreq.set_sensitive(False)
                    self.hourfreq.set_sensitive(False)
            else:
                return
        else:
            return
        schema = Gio.Settings(schema = "org.cdos.update")
        schema.set_boolean("checkupdate", willcheckupdate)

    def setDayFrequecy(self, dayfreq, hourfreq):
        iterator = self.dayfreq_list_store.get_iter_first()
        while iterator:
            if self.dayfreq_list_store[iterator][0] == self.dayfreqArray[dayfreq]:
                if iterator != self.dayfreq.get_active_iter():
                    self.dayfreq.set_active_iter(iterator)
                break
            iterator = self.dayfreq_list_store.iter_next(iterator)

        iterator = self.hourfreq_list_store.get_iter_first()
        while iterator:
            if self.hourfreq_list_store[iterator][0] == (str(hourfreq)+":00"):
                if iterator != self.hourfreq.get_active_iter():
                    self.hourfreq.set_active_iter(iterator)
                break
            iterator = self.hourfreq_list_store.iter_next(iterator)

    # def on_changed_dayfreq(self, combobox):
    def set_dayfreq(self):
        combobox = self.dayfreq
        model = combobox.get_model()
        iterator = combobox.get_active_iter()
        chosen_item = -1
        for i in range(len(self.dayfreqArray)):
            if self.dayfreqArray[i] == model[iterator][0]:
                chosen_item = i
                break
        if chosen_item >= 0:
            schema = Gio.Settings(schema = "org.cdos.update")
            schema.set_int("check-day", chosen_item)

    # def on_changed_hourfreq(self, combobox):
    def set_hourfreq(self):
        combobox = self.hourfreq
        model = combobox.get_model()
        iterator = combobox.get_active_iter()
        schema = Gio.Settings(schema = "org.cdos.update")
        schema.set_int("check-hour", int(model[iterator][0].split(':')[0]))

    def on_changed_updatefrequence(self, combobox):
        if (self.dayfreq.get_sensitive() == False) or (self.hourfreq.get_sensitive() == False):
            self.dayfreq.set_sensitive(True)
            self.hourfreq.set_sensitive(True)
        self.confirm_btn.set_sensitive(True)
        self.cancel_btn.set_sensitive(True)

    def on_changed_dayfreq(self, combobox):
        self.confirm_btn.set_sensitive(True)
        self.cancel_btn.set_sensitive(True)

    def on_changed_hourfreq(self, combobox):
        self.confirm_btn.set_sensitive(True)
        self.cancel_btn.set_sensitive(True)
    
    def on_changed_distupgrade(self, combobox):
        self.confirm_btn.set_sensitive(True)
        self.cancel_btn.set_sensitive(True)

    def on_confirm_press(self, *args):
        self.set_updatefrequence()
        self.set_dayfreq()
        self.set_hourfreq()
        self.set_update_distupgrade()

    def set_update_distupgrade(self):
        combobox = self.distupgrade_list
        model = combobox.get_model()
        iterator = combobox.get_active_iter()
        chosen_item = -1
        for i in range(len(self.distupgrade_list_array)):
            if self.distupgrade_list_array[i] == model[iterator][0]:
                chosen_item = i
                break
        schema = Gio.Settings(schema = "org.cdos.update")
        schema.set_int("distupgrade-opts", chosen_item)

    def set_distupgrade(self, opts):
        chosen_item = self.distupgrade_list_array[opts]

        iterator = self.distupgrade_list_store.get_iter_first()
        while iterator:
            if self.distupgrade_list_store[iterator][0] == chosen_item:
                if iterator != self.distupgrade_list.get_active_iter():
                    self.distupgrade_list.set_active_iter(iterator)
                break
            iterator = self.distupgrade_list_store.iter_next(iterator)

        self.set_update_distupgrade()

    def on_confirm_click(self, button):
        self.confirm_btn.set_sensitive(False)
        self.go_to_main_page()

    def on_cancel_click(self, button):
        schema = Gio.Settings(schema = "org.cdos.update")
        dayfreq = schema.get_int("check-day")
        hourfreq = schema.get_int("check-hour")
        self.setDayFrequecy(dayfreq, hourfreq)
        autoinstall = schema.get_boolean("autoinstall")
        autodownload = schema.get_boolean("autodownload")
        willcheckupdate = schema.get_boolean("checkupdate")
        distupgrade_opts = schema.get_int("distupgrade-opts")
        self.setFrequecy(willcheckupdate, autodownload, autoinstall)
        self.set_distupgrade(distupgrade_opts)
        self.confirm_btn.set_sensitive(False)
        self.go_to_main_page()

class Updates_Object(Gtk.Grid):
    def __init__(self, update_type, image_name, update_name, update_name_msg, show_detail=False, detail_info="", version="", size="", button_callback=None, deb_list=[], history_long_desc=""):
        Gtk.Grid.__init__(self)
        import gettext as gettext1
        gettext1.bindtextdomain('cdosupdate', "/usr/share/locale/")
        gettext1.textdomain('cdosupdate')
        self._ = gettext1.gettext
        self.set_size_request(-1, 84)
        self.button_callback = button_callback
        self.detail_status = False
        self.type = update_type
        self.deb_list = deb_list
        self.ready = False
        self.working = False
        self.detail_info = detail_info
        self.image_name = image_name
        self.update_name = update_name
        self.update_name_msg = update_name_msg
        self.history_long_desc = history_long_desc
        self.version = version
        self.size = size
        self.default_image_name = "/usr/share/cdos-update/daemon/software_center/default.png"

        self.init_object(image_name=image_name, update_name=update_name, update_name_msg=update_name_msg, show_detail=show_detail, detail_info=detail_info,button_callback=button_callback)
        self.show_updates_info( show_detail, button_callback)

    def init_object(self,image_name, update_name, update_name_msg, show_detail, detail_info, button_callback):
        image = Gtk.Image()
        image_wigth = image_height = 54
        if os.path.exists(image_name):
            pixbuf = Pixbuf.new_from_file_at_scale(image_name, width=image_wigth, height=image_height, preserve_aspect_ratio=False)
        else:
            pixbuf = Pixbuf.new_from_file_at_scale(self.default_image_name, width=image_wigth, height=image_height, preserve_aspect_ratio=False)
        image.set_from_pixbuf(pixbuf)
        image.set_margin_top(14)
        image.set_margin_right(30)
        self.attach(image, 0, 0, 1, 2)

        name_label = Gtk.Label(update_name)
        name_label.set_name("update_name")
        name_label.set_margin_top(22)
        name_label.set_xalign(0)
        self.attach(name_label, 1, 0, 2, 1)

        name_msg_label = Gtk.Label(update_name_msg[:19] + "...")
        name_msg_label.set_name("update_name_msg")
        name_msg_label.set_xalign(0)
        self.attach(name_msg_label, 1, 1, 1, 1)

        self.detail_button = Gtk.LinkButton()
        self.detail_button.set_label(self._("details..."))
        self.detail_button.set_name("detail_button")
        self.detail_button.set_halign(Gtk.Align.START)
        self.detail_button.modify_text(Gtk.StateType.NORMAL, Gdk.Color(237*256, 16*256, 218*256))
        self.detail_button.connect("clicked", self.change_detail_status)
        self.attach(self.detail_button, 2, 1, 1, 1)

        alignment = Gtk.Alignment()
        alignment.set_hexpand(True)
        alignment.set_halign(Gtk.Align.FILL)
        self.attach(alignment, 3, 0, 1, 2)

        self.progressbar_info_label = Gtk.Label()
        self.progressbar_info_label.set_text(self._("waiting..."))
        self.progressbar = Gtk.ProgressBar()
        self.progressbar.set_size_request(380,4)
        self.progressbar.set_margin_top(4)
        self.progressbar_box = Gtk.VBox()
        self.progressbar_box.set_margin_top(30)
        self.progressbar_box.pack_start(self.progressbar_info_label, False, False, 0)
        self.progressbar_box.pack_start(self.progressbar, False, False, 0)
        self.attach(self.progressbar_box, 4, 0, 1, 2)

        self.button = Gtk.Button()
        self.button.set_label(self._("Update"))
        self.button.set_name("update_button")
        self.button.set_margin_top(25)
        self.button.set_margin_left(54)
        self.button.set_size_request(100, 32)
        self.button.set_halign(Gtk.Align.END)
        self.button.set_valign(Gtk.Align.START)
        self.attach(self.button, 5, 0, 1, 2)

        self.button.connect("clicked", self.update_exec)

        self.detail_label = Gtk.Label(detail_info)
        self.detail_label.set_name("detail_label")
        self.detail_label.set_xalign(0)
        self.detail_label.set_margin_left(12)
        self.attach(self.detail_label, 1, 2, 4, 1)

        alignment2 = Gtk.Alignment()
        alignment2.set_vexpand(True)
        alignment2.set_valign(Gtk.Align.FILL)
        alignment2.set_halign(Gtk.Align.END)
        alignment2.set_valign(Gtk.Align.START)
        self.attach(alignment2, 0, 3, 6, 1)
        self.separator = Gtk.HSeparator()
        self.separator.set_margin_top(14)
        self.attach(self.separator, 1, 4, 6, 1)

    def change_detail_status(self, *argv):
        if self.detail_status == True:
            self.detail_label.hide()
            self.detail_status = False
            self.detail_button.set_label(self._("details..."))
        else:
            self.detail_label.show()
            self.detail_status = True
            self.detail_button.set_label(self._("hide details"))

    def hide_button(self):
        self.button.set_label("")
        self.button.set_sensitive(False)
        self.button.hide()
        self.progressbar_box.set_margin_right(154)
        if not self.type == "upgrade":
            self.show_progress_box()

    def finish(self):
        for widget in self.get_children():
            widget.destroy()
        self.destroy()

    def set_progress_bar(self, num):
        try:
            self.progressbar_info_label.set_text( str(int(num)) + "%")
            self.progressbar.set_fraction( float(float(num)/100.0))
        except Exception as e:
            logging.error("set progessbar error:" + str(e))
            self.progressbar.set_fraction(0)

    def do_work(self, *argc):
        self.ready = True
        self.button.set_label(self._("Cancel"))
        self.button_callback(self.type, self.deb_list, self, action="update")
        self.show_progress_box()

    def update_exec(self, *argv):
        if self.type == "upgrade":
            self.button_callback()
            return
        else:
            self.show_progress_box()
        if self.button_callback is not None:
            if self.ready == False:
                self.ready = True
                self.button.set_label(self._("Cancel"))
                self.button_callback(self.type, self.deb_list, self, action="update")
                self.show_progress_box()
            else:
                self.ready = False
                self.button.set_label(self._("Update"))
                self.button_callback(self.type, self.deb_list, self, action="cancel")
                self.hide_progress_box()

    def finish_exec(self):
        self.ready = False
        self.show_button()
        self.progressbar.set_fraction( 0 )
        self.progressbar_info_label.set_text(self._("waiting..."))
        self.hide_progress_box()

    def show_updates_info(self, show_detail, button_callback):
        self.show_all()
        self.hide_progress_box()
        self.detail_label.hide()
        if show_detail == False:
            self.detail_button.hide()
        if button_callback is None:
            self.button.hide()

    def set_button_label(self, label_info):
        self.button.set_label(self._(label_info))

    def show_button(self):
        self.button.set_label(self._("Update"))
        self.button.set_sensitive(True)
        self.button.show()
        self.progressbar_box.set_margin_right(0)

    def show_progress_box(self):
        self.progressbar_box.show()

    def hide_progress_box(self):
        self.progressbar_box.hide()

    def set_progress_info(self, info):
        self.progressbar_info_label.set_text(self._(info))

global intern_dbus_name
global intern_dbus_path
global intern_dbus_iface
intern_dbus_name = "org.cdos.update.ui"
intern_dbus_path = "/org/cdos/update/ui"
intern_dbus_iface =  "org.cdos.update.ui"

class OneInstance(dbus.service.Object):

    def __init__(self):
        self.session_bus = dbus.SessionBus()
        name = dbus.service.BusName(intern_dbus_name, bus=self.session_bus)
        dbus.service.Object.__init__(self, name, intern_dbus_path)

    @dbus.service.method(intern_dbus_iface, in_signature='', out_signature='b')
    def show(self):
        return True

if __name__ == "__main__":
    try:
        dbml = DBusGMainLoop(set_as_default=True)
        dbus.set_default_main_loop(dbml)
        bus = dbus.SessionBus()
        remote_object_2 = bus.get_object(intern_dbus_name, intern_dbus_path )
        iface = dbus.Interface(remote_object_2, intern_dbus_iface)
        iface.show()
        exit(0)
    except Exception as e:
        oneinstance = OneInstance()

    img_dir = "/usr/share/cdos-update/frontend/img"
    uidir = "/usr/share/cdos-update/frontend/ui"
    check = CheckMainwindow(img_dir, uidir)

    check.go_to_loading_page()
    check.show()

    GLib.idle_add(check.go_to_updates_page)

    Gtk.main()


